/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.java.net
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.nodes;

import javax.swing.Action;
import org.junit.Test;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;
import static org.hamcrest.CoreMatchers.*;
import org.openide.util.actions.SystemAction;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class DecoratorNodeWithNoActionsTest
  {
    @Test
    public void shouldExposeNoActions()
      {
        final SystemAction action1 = mock(SystemAction.class);
        final SystemAction action2 = mock(SystemAction.class);
        final SystemAction action3 = mock(SystemAction.class);

        final SystemAction[] actions = new SystemAction[] { action1, action2, action3 };

        final Node node = new AbstractNode(Children.LEAF)
          {

            @Override
            protected SystemAction[] createActions()
              {
                return systemActions;
              }

            @Override
            public SystemAction[] getActions()
              {
                return systemActions;
              }

            @Override
            public SystemAction getDefaultAction()
              {
                return action1;
              }

            @Override
            public Action getPreferredAction()
              {
                return action1;
              }

            @Override
            public Action[] getActions(boolean context)
              {
                return systemActions;
              }

            @Override
            public SystemAction[] getContextActions()
              {
                return systemActions;
              }
          };

        final DecoratorNodeWithNoActions noActionsNode = new DecoratorNodeWithNoActions(node);
        assertThat(noActionsNode.getDefaultAction(), is(nullValue()));
        assertThat(noActionsNode.getPreferredAction(), is(nullValue()));
        assertThat(noActionsNode.getActions().length, is(0));
        assertThat(noActionsNode.getActions(true).length, is(0));
        assertThat(noActionsNode.getActions(false).length, is(0));
        assertThat(noActionsNode.getContextActions().length, is(0));
      }
  }