/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.java.net
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.nodes;

import javax.annotation.Nonnull;
import java.util.Arrays;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class GenericDecoratorNodeTest
  {
    private Node node3a;
    private Node node3b;
    private Node node3c;
    private Node node2a;
    private Node node2b;
    private Node rootNode;
 
    public static class FixtureDecoratorNode extends GenericDecoratorNode
      {
        public FixtureDecoratorNode (final @Nonnull Node delegate)
          {
            super(delegate);
          }
      }

    public static class DecoratorNodeWithNonPublicConstructor extends GenericDecoratorNode
      {
        private DecoratorNodeWithNonPublicConstructor (final @Nonnull Node delegate)
          {
            super(delegate);
          }
      }

    @Before
    public void createNodes()
      {
        node3a = createNode("node3a");
        node3b = createNode("node3b");
        node3c = createNode("node3c");

        node2a = createNode("node2a", node3a, node3b);
        node2b = createNode("node2b", node3c);

        rootNode = createNode("node1", node2a, node2b);
      }

    @Test
    public void shouldCreateAFullyDecoratedHierarchy()
      {
        final Node rootDecoratorNode = new FixtureDecoratorNode(rootNode);
        final Node[] decoratorNodes = rootDecoratorNode.getChildren().getNodes();
        assertThat(decoratorNodes.length, is(2));
        assertDecoratorNode(decoratorNodes[0], node2a);
        assertDecoratorNode(decoratorNodes[1], node2b);

        final Node[] decoratorNodes3ab = decoratorNodes[0].getChildren().getNodes();
        assertThat(decoratorNodes3ab.length, is(2));
        assertDecoratorNode(decoratorNodes3ab[0], node3a);
        assertDecoratorNode(decoratorNodes3ab[1], node3b);

        final Node[] decoratorNodes3c = decoratorNodes[1].getChildren().getNodes();
        assertThat(decoratorNodes3c.length, is(1));
        assertDecoratorNode(decoratorNodes3c[0], node3c);
      }

    @Test(expected=Error.class)
    public void shouldThrowExceptionWithNonPublicConstructor()
      {
        new DecoratorNodeWithNonPublicConstructor(rootNode).getChildren().getNodes();
      }

    @Nonnull
    private static Node createNode (final @Nonnull String name, final Node ... children)
      {
        final Node node = new AbstractNode((children.length == 0) ? Children.LEAF : new Children.Array(Arrays.asList(children))
          {
          });
        node.setDisplayName(name);
        return node;
      }

    private static void assertDecoratorNode (final @Nonnull Node decoratorNode, final @Nonnull Node delegate)
      {
        assertThat(decoratorNode, instanceOf(FixtureDecoratorNode.class));
        assertThat(decoratorNode.getDisplayName(), is(delegate.getDisplayName()));
      }
  }