/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.java.net
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package it.tidalwave.netbeans.nodes;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;
import org.junit.Test;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Children.Array;
import org.openide.nodes.Node;
import it.tidalwave.netbeans.nodes.LookupFilterDecoratorNode.LookupFilter;
import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;

/**
 *
 * @author fritz
 */
public class LookupFilterDecoratorNodeTest
  {
    static class FilteredLookup extends ProxyLookup
      {
        public FilteredLookup (final @Nonnull Lookup ... lookups)
          {
            super(lookups);
          }

        @Nonnull
        public Lookup getFilteredLookup()
          {
            return getLookups()[0];
          }
      }

    @Test
    public void mustApplyTheLookupFilterToAllTheNodesInAHierarchy()
      {
        final Lookup[] lookups = new Lookup[15];
        final Node[] nodes = new Node[15];

        for (int i = 0; i< lookups.length; i++)
          {
            lookups[i] = Lookups.fixed("foo" + i); // all different instances
          }

        nodes[0]  = createNode( 0, Children.LEAF, lookups[0]);
        nodes[1]  = createNode( 1, Children.LEAF, lookups[1]);
        nodes[2]  = createNode( 2, Children.LEAF, lookups[2]);
        nodes[3]  = createNode( 3, Children.LEAF, lookups[3]);
        nodes[4]  = createNode( 4, Children.LEAF, lookups[4]);
        nodes[5]  = createNode( 5, Children.LEAF, lookups[5]);
        nodes[6]  = createNode( 6, Children.LEAF, lookups[6]);
        nodes[7]  = createNode( 7, Children.LEAF, lookups[7]);

        nodes[8]  = createNode( 8, children(nodes[0], nodes[1]), lookups[8]);
        nodes[9]  = createNode( 9, children(nodes[2], nodes[3]), lookups[9]);
        nodes[10] = createNode(10, children(nodes[4], nodes[5]), lookups[10]);
        nodes[11] = createNode(11, children(nodes[6], nodes[7]), lookups[11]);

        nodes[12] = createNode(12, children(nodes[8], nodes[9]), lookups[12]);
        nodes[13] = createNode(13, children(nodes[10], nodes[11]), lookups[13]);

        nodes[14] = createNode(14, children(nodes[12], nodes[13]), lookups[14]);

        final LookupFilter lookupFilter = new LookupFilter()
          {
            @Override
            public Lookup filter (final @Nonnull Lookup lookup)
              {
                return new FilteredLookup(lookup);
              }
          };

        final Node filterNode = new LookupFilterDecoratorNode(nodes[14], lookupFilter);
        final List<Node> children = getChildrenRecursively(filterNode);

        for (int i = 0; i < children.size(); i++)
          {
            final Node node = children.get(i);
            assertThat(node, is(instanceOf(LookupFilterDecoratorNode.class)));
            final Lookup nodeLookup = node.getLookup();
            assertThat(nodeLookup, is(instanceOf(FilteredLookup.class)));
            final int index = Integer.parseInt(node.getName());
            assertThat("At position " + index, ((FilteredLookup)nodeLookup).getFilteredLookup(), is(sameInstance(lookups[index])));
          }
      }

    @Nonnull
    private static Children children (final @Nonnull Node ... nodes)
      {
        final Array children = new Children.Array();
        children.add(nodes);
        return children;
      }

    @Nonnull
    private static List<Node> getChildrenRecursively (final @Nonnull Node node)
      {
        final List<Node> result = new ArrayList<Node>();

        for (final Node child : node.getChildren().getNodes())
          {
            result.addAll(getChildrenRecursively(child));
            result.add(child);
          }

        return result;
      }

    @Nonnull
    private static Node createNode (final @Nonnull int index,
                                    final @Nonnull Children children,
                                    final @Nonnull Lookup lookup)
      {
        final AbstractNode node = new AbstractNode(children, lookup);
        final String name = Integer.toString(index);
        node.setName(name);
        node.setDisplayName(name);
        return node;
      }
  }